import kotlinx.datetime.LocalDateTime
import java.awt.image.BufferedImage
import java.util.*
import kotlin.collections.HashMap

class AppModel {
    var version: Long = 0
        private set
    private var tasks: HashMap<UUID, TaskRecord> = HashMap()

    fun updateTask(task: TaskRecord): TaskRecord? { // returns the old version
        version += 1
        val old = tasks[task.uuid]
        tasks[task.uuid] = task
        return old
    }

    fun getTask(uuid: UUID): TaskRecord? {
        return tasks[uuid]
    }

    fun removeTask(uuid: UUID): TaskRecord? {
        version += 1
        return tasks.remove(uuid)
    }

    fun getTasksByBlinkAt(): ArrayList<TaskRecord> {
        val list = ArrayList<TaskRecord>()
        for (t in tasks) { list.add(t.value) }
        list.sortBy { t -> t.blinkAt }
        return list
    }
}

class TaskRecord(
    val uuid: UUID,
    val icon: IconRecord,
    val task: String,
    val link: LinkRecord?,
    val createdAt: LocalDateTime,
    val blinkAt: LocalDateTime,
)

class LinkRecord(
    val icon: IconRecord,
    val window: Int,
    val pid: Int,
    val filename: String,
)

class IconRecord(val image: BufferedImage) {
    val cachedButtonArt = ButtonArt.fromArbitraryImage(image)  // don't persist this, just have it always ready
}