import java.awt.*
import javax.swing.BoxLayout
import javax.swing.JDialog
import javax.swing.JPanel
import javax.swing.border.EmptyBorder

private val GRID_SIZE = 64;
private val GRID_SIZE_INNER = 56;
private val GRID_INSET = (GRID_SIZE - GRID_SIZE_INNER) / 2

class AppView(model: AppModel): JDialog() {
    private val viewModel = AppViewModel(model)

    private val controlPanel = buildControlPanel()
    private val iconPanel = JPanel()
    // TODO: Replace these with icons that have clear meaning

    init {
        isAlwaysOnTop = true
        isUndecorated = true
        background = TRANSPARENT
        bounds = Rectangle(0, 0, Toolkit.getDefaultToolkit().screenSize.width, 64)

        contentPane.layout = BorderLayout()
        contentPane.add(controlPanel, BorderLayout.WEST)
        contentPane.add(iconPanel, BorderLayout.CENTER)

        iconPanel.layout = FlowLayout(FlowLayout.LEFT, 0, 0)

        controlPanel.background = TRANSPARENT;
        iconPanel.background = TRANSPARENT;
    }

    fun touch() {
        viewModel.ifDirty(contentPane) {
            val ipHeight = iconPanel.height
            val iconSize = ipHeight * 3/4;
            val meterHeight = ipHeight - iconSize;

            iconPanel.removeAll()

            val tasks = viewModel.model.getTasksByBlinkAt()
            for (task in tasks) {
                iconPanel.add(TaskView(task))
            }
        }
    }

    private fun buildControlPanel(): JPanel {
        val moveButton = ImageButton(
            ButtonArt(
                getImage("/move_unlit.png")!!,
                getImage("/move_lit.png")!!,
                getImage("/move_pressed.png")!!,
            ),
            null
        )
        // TODO: Do something
        moveButton.preferredSize = Dimension(GRID_SIZE, GRID_SIZE)
        moveButton.border = EmptyBorder(GRID_INSET, GRID_INSET, GRID_INSET, GRID_INSET)
        moveButton.callback = { print("test") }

        val controlPanel = JPanel()
        controlPanel.layout = BoxLayout(controlPanel, BoxLayout.PAGE_AXIS)

        controlPanel.add(moveButton)

        return controlPanel
    }
}

// for now: just reexpose the underlying model
// we don't have a layer of state beyond what can be kept in the UI
class AppViewModel(val model: AppModel) {
    private var cpHeight: Int = -1
    private var version: Long = -1

    fun ifDirty(contentPane: Container, body: () -> Unit) {
        if (version != model.version || contentPane.height != cpHeight) {
            body()
            cpHeight = contentPane.height
            version = model.version
        }
    }
}

class TaskView(
    private val task: TaskRecord,
): JPanel() {
    private var button: ImageButton
    init {
        layout = BorderLayout()
        preferredSize = Dimension(GRID_SIZE, GRID_SIZE)
        border = EmptyBorder(GRID_INSET, GRID_INSET, GRID_INSET, GRID_INSET)
        background = TRANSPARENT

        button = ImageButton(
            task.icon.cachedButtonArt,
            null
        )

        add(button)
    }

    /*
    override fun paintComponent(gnull: Graphics?) {
        super.paintComponent(gnull)

        val g = gnull!!

        // icon occupies top of space
        g.drawImage(task.icon.image, 0, 0, iconSize, iconSize, Color(0, 0, 0, 0)) { _, _, _, _, _, _ -> false }

        // bar occupies middle 1/2 of space
        val realBarHeight = meterHeight / 2
        val barY = iconSize + (meterHeight - realBarHeight)/2
        val arcSize = realBarHeight / 2
        val barX = arcSize
        val realBarWidth = iconSize - arcSize * 2

        g.color = Color(0, 255, 0, 255)
        g.fillRoundRect(barX, barY, realBarWidth, realBarHeight, arcSize, arcSize)
    }
     */
}