import java.awt.Dimension
import java.awt.Graphics
import java.awt.event.MouseAdapter
import java.awt.event.MouseEvent
import java.awt.event.MouseListener
import java.awt.image.BufferedImage
import javax.swing.JPanel

class ImageButton(
    private val art: ButtonArt,
    var callback: (() -> Unit)?,
): JPanel() {
    private var mouseOver = false
    private var mouseDown = false

    init {
        background = TRANSPARENT
        preferredSize = Dimension(art.base.width, art.base.height)
        this.addMouseListener(object: MouseAdapter() {
            override fun mouseEntered(e: MouseEvent?) {
                super.mouseEntered(e)
                mouseOver = true
                repaint()
            }

            override fun mouseExited(e: MouseEvent?) {
                super.mouseExited(e)
                mouseOver = false
                repaint()
            }

            override fun mousePressed(e: MouseEvent?) {
                super.mousePressed(e)
                mouseDown = true
                repaint()
            }

            override fun mouseReleased(e: MouseEvent?) {
                super.mouseReleased(e)
                mouseDown = false
                repaint()
            }

            override fun mouseClicked(e: MouseEvent?) {
                super.mouseClicked(e)
                callback?.invoke()
            }
        })
    }

    override fun paintComponent(gnull: Graphics?) {
        super.paintComponent(gnull)

        val g = gnull!!
        val img = if (mouseDown) { art.pressed } else if (mouseOver) { art.highlighted } else { art.base }

        val insets = insets
        val dx = width - (insets.left + insets.right)
        val dy = height - (insets.top + insets.bottom)

        g.drawImage(img, insets.left, insets.top, dx, dy, null)
    }
}